/*
    GPSUTIL a program to interact with NMEA compatable and other supported
      GPS units.

    Copyright (C) 2000  Brian J. Hennings

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    I can be contacted at hennings@cs.uakron.edu or by US mail at:

    Brian Hennings
    114 Rex Ave.
    Wintersville, OH 43953
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef LINUX
#include <sys/signal.h>
#endif
#ifdef WIN32
#include <windows.h>
#endif
#include "serial.h"
#include "gps.h"
#include "nmea.h"
#include "magellan.h"

CPORT Serial = 0;
int   debug = 0;

#ifdef LINUX
/*****************************************************************************
 * SigHndlr () - A very simple signal handler that closes the serial port
 *     before the program terminates.
 *****************************************************************************/
void SigHndlr (int Val)
{
 ClosePort ();
 exit (-1);
} /*End of SigHndlr ()*/
#endif

/*****************************************************************************
 * GetOpt () - Used to handle command line args.  valstring is a simple
 *      string containing the valid characters.  If a character is followed
 *      by a colon it means the next item in argv is to be passed back using
 *      optarg.  There is little error protection in this routine.  If a
 *      command line arguement needs an argument passed back, the next item in
 *      argv is sent back.  If there isn't another argv then a null string is
 *      passed back.
 *****************************************************************************/
char GetOpt (int argc, char *argv[], char *valstring, char *optarg)
{static int Loc = 1;
        int Offset = 0,
            Idx,
            Tmp;
       char RetVal;

 optarg[0] = '\0';  /*Just in case!!!*/

 if (Loc >= argc)
   return EOF;

 if (argv[Loc][Offset] == '-')
   Offset++;

 RetVal = argv[Loc][Offset];
 Tmp = strlen (valstring);

 for (Idx = 0; Idx < Tmp; Idx++)
   if (valstring[Idx] == RetVal)
     break;

 if (valstring[Idx] == RetVal) {
   if (valstring[Idx + 1] == ':') {
     if ((Loc + 1) < argc)
       strcpy (optarg, argv[Loc + 1]);
     Loc++;
   } /*End of if*/
 } /*End of if*/
 else
   RetVal = '?';

 Loc++;

 return RetVal;
} /*End of GetOpt*/

/********/
/* MAIN */
/********/
int main (int argc, char *argv[])
{char  SPort[MAXLEN] = CPRT,
       copt,
      *delwpt,
       Name[20] = "",
       UFile[MAXLEN], DFile[MAXLEN],
       optarg[MAXLEN],
       opUL = 0, opDL = 0, opPOS = 0, opWInfo = 0;
 int   errflag = 0;
 float Speed = 0, Heading = 0, Range = 0;
 MWpt  WList;
 POS   CurCoords;
 char  elist[MAXLEN];

#ifdef LINUX
 signal (SIGINT, SigHndlr);  //Close serial port BEFORE terminating on a control-c
#endif

 printf ("GPSUTIL version %s, Copyright (C) 2000 Brian J. Hennings\n", VERSION);
 printf ("GPSUTIL comes with ABSOLUTELY NO WARRANTY; for details see license.txt.\n\n");
 memset (&WList, '\0', sizeof (MWpt));
 memset (&CurCoords, '\0', sizeof (POS));

 while ((copt = GetOpt(argc, argv, "p:D:d:e:u:gwvV", optarg)) != EOF) {
   switch (copt) {
     case 'p':
       strcpy (SPort, optarg);
       if (optarg[0] == '\0')
         errflag++;
     break;
     case 'v': case 'V':
     break;
     case 'g':
       opPOS = 1;
     break;
     case 'w':
       opWInfo = 1;
     break;
     case 'D':
       if (optarg[0] == '\0')
         debug++;
       else
         debug = atoi (optarg);
       printf ("%s (%s): debug level %d enabled.\n", argv[0], VERSION, debug);
     break;
     case 'd':
       if (optarg[0] == '\0')
         errflag++;
       opDL = 1;
       strcpy (DFile, optarg);
     break;
     case 'u':
       if (optarg[0] == '\0')
         errflag++;
       opUL = 1;
       strcpy (UFile, optarg);
     break;
     case 'e':
       strcpy(elist, optarg);
     break;
     case '?':
       errflag += 2;
     break;
   } /*End of switch */
 } /*End of while*/

 if (errflag > 0) {
   printf ("\nUSAGE: %s [-options]\n\n", argv[0]);
   printf ("Options:\n");
   printf ("-?                 Display this help message\n");
   printf ("-v                 Print Version information.\n");
   printf ("-p Port            Serial Port Name                   -- default '%s'\n", CPRT);
   printf ("-D [Level]         Set Debugging level                -- default 0\n");
   printf ("-g                 Get current latitude / longitude (NMEA)\n");
   printf ("-w                 Heading and range to current waypoint (NMEA)\n");
   printf ("-d FILENAME        Download waypoint list to FILENAME -- NO default\n");
   printf ("-u FILENAME        Upload waypoint list from FILENAME -- NO default\n");
   printf ("-e WPT1[,WPT2,...] Delete waypoints\n");
   return -1;
 } /*End of if*/

 if (OpenPort (SPort) != 0) {
   printf ("Unable to open serial port (%s)\n", SPort);
   return -1;
 } /*End of if*/

 if (opDL) {
   if (MagellanInit()) {
     ClosePort ();
     return -1;
   } /*End of if*/
   MagellanDLWPts (&WList);
   MDumpWPts(DFile, &WList);
   MFreeWPts (&WList);
 } /*End of if*/

 if (opUL) {
   if (MagellanInit()) {
     ClosePort ();
     return -1;
   } /*End of if*/
   MRestoreWPts (UFile);
 } /*End of if*/

 if (opWInfo) {
   ReadPos (&CurCoords);
   printf ("Curent ");
   printf ("Latitude: %02d %02d %02.03f%c Longitude: %02d %02d %02.03f%c\n",
           CurCoords.Latitude.Hours,
           CurCoords.Latitude.Minutes, CurCoords.Latitude.Seconds,
           CurCoords.Latitude.Dir, CurCoords.Longitude.Hours,
           CurCoords.Longitude.Minutes, CurCoords.Longitude.Seconds,
           CurCoords.Longitude.Dir);
   ReadWInfo (Name, 20, &CurCoords, &Heading, &Range);
   printf ("%s ", Name);
   printf ("Latitude: %02d %02d %02.03f%c Longitude: %02d %02d %02.03f%c\n",
           CurCoords.Latitude.Hours,
           CurCoords.Latitude.Minutes, CurCoords.Latitude.Seconds,
           CurCoords.Latitude.Dir, CurCoords.Longitude.Hours,
           CurCoords.Longitude.Minutes, CurCoords.Longitude.Seconds,
           CurCoords.Longitude.Dir);
   printf ("Bearing: %05.1f Range: %05.1f\n", Heading, Range);
 } /*End of if*/

 if (opPOS) {
   ReadPos (&CurCoords);
   printf ("Latitude: %02d %02d %02.03f%c Longitude: %02d %02d %02.03f%c\n",
           CurCoords.Latitude.Hours,
           CurCoords.Latitude.Minutes, CurCoords.Latitude.Seconds,
           CurCoords.Latitude.Dir, CurCoords.Longitude.Hours,
           CurCoords.Longitude.Minutes, CurCoords.Longitude.Seconds,
           CurCoords.Longitude.Dir);
   ReadSnH (&Heading, &Speed);
   printf ("Speed: %06.2f Heading: %05.1f\n", Speed, Heading);
 } /*End of if*/

 if (elist[0]) {
   MagellanHandOn();
   for (delwpt = strtok(elist, ","); delwpt; delwpt = strtok(NULL, ",")) {
     MagellanDelWPT(delwpt);
   } /*End of for*/
   MagellanHandOff();
 } /*End of if*/

 ClosePort ();

 return 0;
} /*End of main ()*/

