/*
    GPSUTIL a program to interact with NMEA compatable and other supported
      GPS units.

    Copyright (C) 2000  Brian J. Hennings

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    I can be contacted at hennings@cs.uakron.edu or by US mail at:

    Brian Hennings
    114 Rex Ave.
    Wintersville, OH 43953
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef WIN32
#include <windows.h>
#endif

#include "gps.h"
#include "nmea.h"
#include "serial.h"

extern CPORT Serial;
extern int   debug;

/************************************************************************
 * GetParam () - Returns the parameter at the specified position.
 ************************************************************************/
int GetParam (int PPos, char *Value, int MaxLen, char *Msg)
{int   idx = 0;
 char *cur = NULL;

 memset (Value, '\0', MaxLen);
 cur = Msg;
 for (idx = 0; idx < PPos; idx++) {
   cur = strchr (cur, ',');
   if (cur == NULL)
     return -1;
   cur++;
 } /*End of for*/

 idx = 0;
 while ((cur[idx] != ',')  && (cur[idx] != '*') &&
	(cur[idx] != '\0') && (idx < MaxLen)) {
   Value[idx] = cur[idx];
   idx++;
 } /*End of while*/

 return 0;
} /*End of GetParam ()*/

/************************************************************************
 * GLLtoPos () - Converts the NMEA message GLL (current coordinates) to
 *     a structure that contains the data.
 ************************************************************************/
static void GLLtoPos (char *Message, POS *coords)
{double tmp1, tmp2;

 sscanf(Message, "$GPGLL,%le,%c,%le,%c", &tmp1, &coords->Latitude.Dir,
        &tmp2,&coords->Longitude.Dir);

 coords->Latitude.Hours = tmp1 / 100;
 coords->Latitude.Minutes = tmp1 - coords->Latitude.Hours * 100;
 coords->Latitude.Seconds = tmp1 - coords->Latitude.Hours * 100;
 coords->Latitude.Seconds = ((tmp1 - (int)tmp1)) * 60;

 coords->Longitude.Hours = tmp2 / 100;
 coords->Longitude.Minutes = tmp2 - coords->Longitude.Hours * 100;
 coords->Longitude.Seconds = tmp2 - coords->Longitude.Hours * 100;
 coords->Longitude.Seconds = ((tmp2 - (int)tmp2)) * 60;

 if (debug > 1) {
   printf ("Hours: %02d Minutes: %02d Seconds: %02.04f %c\n",
           coords->Latitude.Hours, coords->Latitude.Minutes,
           coords->Latitude.Seconds, coords->Latitude.Dir);
   printf ("by\nHours: %02d Minutes: %02d Seconds: %02.04f %c\n",
           coords->Longitude.Hours, coords->Longitude.Minutes,
           coords->Longitude.Seconds, coords->Longitude.Dir);
 } /*End of if*/

} /*End of GLLtoPos ()*/

/************************************************************************
 * ReadPos () - Attempts to find an NMEA current position message and
 *     calls GLLtoPos to convert the message into a structure.
 ************************************************************************/
int ReadPos (POS *Coord)
{int  rc;
 char Msg[MAXLEN];

 rc = FindMessage ("$??GLL", Msg, MAXLEN);
 if (rc != 0)
   return -1;

 GLLtoPos (Msg, Coord);

 return 0;
} /*End of ReadPos ()*/

/************************************************************************
 * ReadSnH - Read the RMC NMEA message.  This message heading, and speed
 *     in knots.
 ************************************************************************/
int ReadSnH (float *Heading, float *Speed)
{int  rc;
 char Msg[MAXLEN],
      Tmp[10];

 rc = FindMessage ("$??RMC", Msg, MAXLEN);
 if (rc != 0)
   return -1;

 GetParam (7, Tmp, 10, Msg);
 *Speed = atof (Tmp) * 1.15078 ;
 GetParam (8, Tmp, 10, Msg);
 *Heading = atof (Tmp);

 return 0;
} /*End of ReadSnH ()*/

/************************************************************************
 * ReadWInfo - Read the RMB NMEA message.  This message is only sent
 *     when the GPS has a waypoint set as a destination.  The message
 *     contains the waypoint, bearing to waypoint, and range to
 *     waypoint.
 ************************************************************************/
int ReadWInfo (char *Name, int NLen, POS *coords, float *Heading,
               float *Range)
{int  rc, idx, Len;
 char Msg[MAXLEN],
      Tmp[20],
      Tmp2[20];

 rc = FindMessage ("$??RMB", Msg, MAXLEN);
 if (rc != 0)
   return -1;

 GetParam (5, Tmp, 20, Msg);
 if ((strlen (Tmp)) > NLen)
   Name[0] = '\0';
 else
   strcpy (Name, Tmp);

 GetParam (6, Tmp, 20, Msg);
 Tmp2[0] = Tmp[0];
 Tmp2[1] = Tmp[1];
 Tmp2[3] = '\0';
 coords->Latitude.Hours = atoi (Tmp2);
 Len = strlen (Tmp) + 1;    //Include the NULL byte!
 for (idx = 2; idx < Len; idx++)
   Tmp[idx - 2] = Tmp[idx];
 Tmp2[0] = Tmp[0];
 Tmp2[1] = Tmp[1];
 coords->Latitude.Minutes = atoi (Tmp2);
 Len = strlen (Tmp) + 1;    //Include the NULL byte!
 for (idx = 3; idx < Len; idx++)
   Tmp[idx - 3] = Tmp[idx];
 coords->Latitude.Seconds = ((atoi (Tmp) * 1.0) / 10000) * 60;
 GetParam (7, Tmp, 20, Msg);
 coords->Latitude.Dir = Tmp[0];
 GetParam (8, Tmp, 20, Msg);
 Tmp2[0] = Tmp[0];
 Tmp2[1] = Tmp[1];
 Tmp2[2] = Tmp[2];
 Tmp2[3] = '\0';
 coords->Longitude.Hours = atoi (Tmp2);
 Len = strlen (Tmp) + 1;    //Include the NULL byte!
 for (idx = 3; idx < Len; idx++)
   Tmp[idx - 3] = Tmp[idx];
 Tmp2[0] = Tmp[0];
 Tmp2[1] = Tmp[1];
 Tmp2[2] = '\0';
 coords->Longitude.Minutes = atoi (Tmp2);
 Len = strlen (Tmp) + 1;    //Include the NULL byte!
 for (idx = 3; idx < Len; idx++)
   Tmp[idx - 3] = Tmp[idx];
 coords->Longitude.Seconds = ((atoi (Tmp) * 1.0) / 10000) * 60;

 GetParam (9, Tmp, 20, Msg);
 coords->Longitude.Dir = Tmp[0];

 GetParam (10, Tmp, 20, Msg);
 *Range = atof (Tmp);
 GetParam (11, Tmp, 20, Msg);
 *Heading = atof (Tmp);

 return 0;
} /*End of ReadWInfo ()*/

